package com.kameleoon.kameleoon_client_flutter

import org.json.JSONArray
import org.json.JSONObject

fun JSONArray.toList(): List<Map<String, *>> = 0.until(this.length()).map {
    when (val value = this[it]) {
        is JSONObject -> value.toMap()
        else -> emptyMap<String, Nothing>()
    }
}

fun JSONObject.toMap(): Map<String, *> = keys().asSequence().associateWith {
    when (val value = this[it]) {
        is JSONArray -> {
            val map = 0.until(value.length()).associate { Pair(it.toString(), value[it]) }
            JSONObject(map).toMap().values.toList()
        }
        is JSONObject -> value.toMap()
        JSONObject.NULL -> null
        else            -> value
    }
}

fun Any.toFlutterResult(): Any {
    return when (this) {
        is JSONArray -> toList()
        is JSONObject -> toMap()
        is Collection<*> -> mapNotNull { it?.toFlutterResult() }
        is Map<*, *> -> entries.associate { it.key to it.value?.toFlutterResult() }
        else -> this
    }
}


interface RawRepresentable<T> {
    val value: T
}

inline fun <reified E, T> valueOf(value: T): E? where E : Enum<E>, E: RawRepresentable<T> {
    return enumValues<E>().firstOrNull { it.value == value }
}

inline fun <reified T : Enum<T>> valueOf(type: String): T? {
    return try {
        java.lang.Enum.valueOf(T::class.java, type)
    } catch (e: IllegalArgumentException) {
        null
    }
}