
import 'package:flutter_test/flutter_test.dart';
import 'package:integration_test/integration_test.dart';
import 'package:kameleoon_client_flutter/src/kameleoon_client.dart';
import 'package:kameleoon_client_flutter/src/kameleoon_client_factory.dart';
import 'package:kameleoon_client_flutter/src/kameleoon_configuration.dart';

import 'constants.dart';
import 'utils.dart';

void main() {
  const String featureFlagKey = "feature_flag_multi_environment_test";

  IntegrationTestWidgetsFlutterBinding.ensureInitialized();

  testEnvironment(
      String? environment,
      String featureKey,
      String expectedVariationKey) async {

    var config = KameleoonConfiguration("clientId", "clientSecret", environment);
    KameleoonClientFactory.forget(SITECODE);
    KameleoonClient client = KameleoonClientFactory.create(SITECODE, config);
    await testAPI(client, () async {
      try {
        bool result = await client.isFeatureActive(VISITOR_CODE, featureKey);
        expect(result, equals(true));

        String expectedFeatureValue = environment ?? "production";

        String variationKey = await client.getFeatureVariationKey(VISITOR_CODE, featureKey);
        expect(variationKey, equals(expectedVariationKey));

        String variableValue = await client.getFeatureVariable(VISITOR_CODE, featureKey, "Key") as String;
        expect(expectedFeatureValue, equals(variableValue.toLowerCase()));
      } catch (e) {
        fail(e.toString() +  ", No exception expected");
      }
    });
  }

  testWidgets("testProduction", (WidgetTester tester) async {
    await testEnvironment(null, featureFlagKey, "on");
  });

  testWidgets("testStaging", (WidgetTester tester) async {
    await testEnvironment("staging", featureFlagKey, "on_stage");
  });

  testWidgets("testDevelopment", (WidgetTester tester) async {
    await testEnvironment("development", featureFlagKey, "on_development");
  });
}
