import 'package:flutter/services.dart';
import 'package:kameleoon_client_flutter/src/kameleoon_configuration.dart';

import 'kameleoon_client.dart';
import 'internal/kameleoon_client_impl.dart'
    if (dart.library.js) 'internal/kameleoon_client_impl_web.dart';

/// [KameleoonClientFactory] to create [KameleoonClient] instance depending of siteCode
abstract class KameleoonClientFactory {
  static const MethodChannel _channel = const MethodChannel("com.kameleoon.flutter");
  static final Map<String, KameleoonClientImpl> _clients = {};

  /// Create or return the instance of [KameleoonClient] depending of the siteCode
  static KameleoonClient create(String siteCode, KameleoonConfiguration config) {
    if (!_clients.containsKey(siteCode)) {
      final instance = KameleoonClientImpl(siteCode, config, _channel);
      _channel.setMethodCallHandler(_nativeMethodCallHandler);
      _clients[siteCode] = instance;
    }
    return _clients[siteCode]!;
  }

  static void forget(String siteCode) {
    _clients.remove(siteCode)?.forget();
  }

  static Future<void> _nativeMethodCallHandler(MethodCall call) async {
    final siteCode = (call.arguments as Map)['siteCode'];
    _clients[siteCode]?.nativeMethodCallHandler(call);
  }
}
