import 'package:flutter/foundation.dart';

/// Abstract class Data for all tracking data
abstract class Data {
  static const TypeCustomData = "CustomData";
  static const TypePageView = "PageView";
  static const TypeConversion = "Conversion";
  static const TypeBrowser = "Browser";
  static const TypeDevice = "Device";

  late String dataType;
}

/// Uses for tracking any type of data
class CustomData implements Data {
  int id;
  List<String> values;
  String dataType = Data.TypeCustomData;

  CustomData(this.id, String value): values = [value];
  CustomData.many(this.id, this.values);
}
/// Uses for tracking page (url, title , referrers)
class PageView implements Data {
  String url;
  String title;
  List<int> referrers;
  String dataType = Data.TypePageView;

  PageView(this.url, this.title, this.referrers);
}

/// Uses for tracking conversion(goalId, revenue, negative)
class Conversion implements Data {
  int goalId;
  double revenue;
  bool negative;
  String dataType = Data.TypeConversion;

  Conversion(this.goalId, {this.revenue = 0, this.negative = false});
}

enum Browsers { CHROME, INTERNET_EXPLORER, FIREFOX, SAFARI, OPERA, OTHER }

/// Uses for tracking browser client
class Browser implements Data {
  int browser;
  String dataType = Data.TypeBrowser;

  Browser(Browsers browsers): browser = browsers.index;
}

enum Devices { PHONE, TABLET, DESKTOP }

/// Uses for tracking browser client
class Device implements Data {
  String device;
  String dataType = Data.TypeDevice;

  Device(Devices device): device = describeEnum(device);
}
