/// Exception indicating that site of experiment is disabled
class SiteCodeDisabled implements Exception {
  late String _message;

  SiteCodeDisabled([String? message]) {
    this._message = message ?? 'Site code disabled';
  }

  @override
  String toString() {
    return _message;
  }
}

/// Exception indicating that the the visitor code is empty or longer than 255 chars
class VisitorCodeNotValid implements Exception {
  late String _message;

  VisitorCodeNotValid([String? message]) {
    this._message = message ?? 'Visitor code is empty or longer than 255 chars';
  }

  @override
  String toString() {
    return _message;
  }
}

/// Exception indicating that the request experiment ID has not been found in the internal configuration of the SDK.
/// This is usually normal and means that the experiment has not yet been started on Kameleoon's side
/// (but code triggering / implementing variations is already deployed on the mobile app's side).
class ExperimentNotFound implements Exception {
  late String _message;

  ExperimentNotFound([String? message]) {
    this._message = message ?? 'Experiment configuration not found';
  }

  @override
  String toString() {
    return _message;
  }
}

/// Exception indicating that the requested feature ID has not been found in the internal configuration of the SDK.
/// This is usually normal and means that the feature flag has not yet been activated on Kameleoon's side
/// (but code implementing the feature is already deployed on the web-application's side).
class FeatureNotFound implements Exception {
  late String _message;

  FeatureNotFound([String? message]) {
    this._message = message ??  'Feature configuration not found';
  }

  @override
  String toString() {
    return _message;
  }
}

/// Exception indicating that the requested variation ID has not been found in the internal configuration of the SDK.
/// This is usually normal and means that the variation's corresponding experiment has not yet been activated
/// on Kameleoon's side.
class VariationNotFound implements Exception {
  late String _message;

  VariationNotFound([String? message]) {
    this._message = message ?? 'Variation configuration not found';
  }

  @override
  String toString() {
    return _message;
  }
}

/// Exception indicating that the requested variable key has not been found in the internal configuration of the SDK.
/// This is usually normal and means that the variation's corresponding experiment has not yet been activated
/// on Kameleoon's side.
class VariableNotFound implements Exception {
  late String _message;

  VariableNotFound({String? message, String? variableKey}) {
    this._message = message ?? 'Variable with key: $variableKey not found';
  }

  @override
  String toString() {
    return _message;
  }
}

/// Exception indicating that the current user did not trigger the required targeting conditions for this experiment.
/// The targeting conditions are defined via Kameleoon's segment builder.
class NotTargeted implements Exception {
  late String _message;

  NotTargeted([String? message]) {
    this._message = message ?? 'Not Targeted';
  }

  @override
  String toString() {
    return _message;
  }
}

/// Exception indicating that the current user triggered the experiment (met the targeting conditions), but
/// did not activate it. The most common reason for that is that part of the traffic has been excluded
/// from the experiment and should not be tracked.
class NotAllocated implements Exception {
  late String _message;

  NotAllocated([String? message]) {
    this._message = message ?? 'Not Allocated';
  }

  @override
  String toString() {
    return _message;
  }
}

/// Exception indicating that the SDK has not completed its initialization yet.
class SDKNotReady implements Exception {
  late String _message;

  SDKNotReady([String? message]) {
    this._message = message ?? 'SDK Not Ready';
  }

  @override
  String toString() {
    return _message;
  }
}

/// Exception indicating that credentials are not provided or invalid
class CredentialsNotFound implements Exception {
  late String _message;

  CredentialsNotFound([String? message]) {
    this._message = message ?? 'Credentials not found';
  }

  @override
  String toString() {
    return _message;
  }
}
